import socket
import json
import time
import sys
import math
from src.constants import *
from src.gamestate import GameState
from _thread import *

class ServerConnection():
	"""
	This class creates the socket for the server. It binds the server IP addressed and the port to create the socket.

	"""
	def __init__(self,server_ip=""):
		if server_ip == "":
			self.server_addr = SERVER_IP
		else:
			self.server_addr = server_ip
	
		self.server_port = SERVER_PORT
		self.socket_address = (self.server_addr,self.server_port)
		self.server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	
	def create_connection(self):
		try:
			self.server_socket.bind(self.socket_address)
			self.server_socket.listen()
			print(f"Server running at IP: {self.server_addr} and PORT: {self.server_port}")
			print("Server listening now...")
		except socket.error as e:
			print(str(e))
			print("Server could not start due to some error! See Ya!")
			quit()


def resetGameState(gameState):
	"""
	This function resets gameState to reset the game lobby when no clients are connected to the server.

	Parameters
		----------
		 gameState : Dictionary
            This dictionary contains complete information about all the clients connected to the server.

	Return
		----------
		None
	"""
	gameState.food_info=[]
	generate_food(random.randrange(200,250),gameState)
	gameState.running = True
	gameState.cur_time = 0
	gameState.start_time = time.time()

def starting_pos(gameState):
	"""
	This function will calculate the x and y co-ordinate of the position where the client will spawn. The logic ensures that no client spawns inside the radius of an already existing client.

	Parameters
		----------
		gameState : Dictionary
            This dictionary contains complete information about all the clients connected to the server.

	Return
		----------
		tupple(x,y) : int
			The calculated spawning co-ordinates of the client.
	"""
	running = True
	while running:
		running = False
		x = random.randrange(0,WINDOW_WIDTH)
		y = random.randrange(0,WINDOW_HEIGHT)
		for key,value in gameState.connectedClients.items():
			center_distance = math.sqrt((value["x"]-x)**2 + (value["y"]-y)**2)
			if center_distance <= gameState.player_radius + value["radius"]:
				running = True
	return (x,y)

def generate_food(food_points,gameState):
	"""
	This function will generate the circles/food on the screen. The logic guarantees that each generated circle will not spawn inside any connected client.

	Parameters
		----------
		food_points : int
			The number of circles that have to be generated.

		gameState : Dictionary
            This dictionary contains complete information about all the clients connected to the server.

	Return
		----------
		None
	"""
	for i in range(food_points):
		running = True
		while running:
			running = False
			food_x = random.randrange(0,WINDOW_WIDTH)
			food_y = random.randrange(0,WINDOW_HEIGHT)
			for key,value in gameState.connectedClients.items():
				center_distance = math.sqrt((food_x - value["x"])**2 + (food_y-value["y"])**2)
				if center_distance <= gameState.food_radius + value["radius"]:
					running = True
		gameState.food_info.append([food_x,food_y, random.choice(COLOR)])

def reduce_radius(gameState):
	"""
	This function will reduce the radius of all the clients by 0.05%

	Parameters
		----------
		gameState : Dictionary
            This dictionary contains complete information about all the clients connected to the server.

	Return
		----------
		None
	"""
	for key,value in gameState.connectedClients.items():
		if value["radius"] > 10:
			value["radius"] = math.floor(value["radius"]*0.95)

def player_eats_food(gameState):
	"""
	This function will check for collision between any client and the generated food point. If a collision is found then the particular food item will be removed and the collided player's radius will increase.

	Parameters
		----------
		gameState : Dictionary
            This dictionary contains complete information about all the clients connected to the server.

	Return
		----------
		None
	"""
	for key,value in gameState.connectedClients.items():
		player_x = value["x"]
		player_y = value["y"]
		for food_item in gameState.food_info:
			center_distance = math.sqrt((player_x - food_item[0])**2 + (player_y-food_item[1])**2)
			if center_distance <= gameState.food_radius + value["radius"]:
				value["radius"] = value["radius"] + 0.5
				gameState.food_info.remove(food_item)


def player_eats_player(gameState):
	"""
	This function will check for collision between any two pair of clients. If a collision is detected then the client with higher radius/score absorbs the client with lower radius and it's radius increases. The client with lower radius/score is respawned with starting radius.

	Parameters
		----------
		gameState : Dictionary
            This dictionary contains complete information about all the clients connected to the server.

	Return
		----------
		None
	"""
	sorted_players = sorted(gameState.connectedClients, key=lambda x: gameState.connectedClients[x]["radius"])
	for x, player_1 in enumerate(sorted_players):
		for player_2 in sorted_players[x+1:]:
			player_1_x = gameState.connectedClients[player_1]["x"]
			player_1_y = gameState.connectedClients[player_1]["y"]

			player_2_x = gameState.connectedClients[player_2]["x"]
			player_2_y = gameState.connectedClients[player_2]["y"]

			center_distance = math.sqrt((player_1_x - player_2_x)**2 + (player_1_y-player_2_y)**2)
			if center_distance < gameState.connectedClients[player_2]["radius"] - gameState.connectedClients[player_1]["radius"]*0.85:
				gameState.connectedClients[player_2]["radius"] = math.sqrt(gameState.connectedClients[player_2]["radius"]**2 + gameState.connectedClients[player_1]["radius"]**2) 
				gameState.connectedClients[player_1]["radius"] = 0
				gameState.connectedClients[player_1]["x"], gameState.connectedClients[player_1]["y"] = starting_pos(gameState)

def connect_client(client_socket, player_id,client_address,gameState):
	"""
	This function runs in a new thread on the server. It adds a new client to the gameState and continously retrieves the updates of the clients. It also sends the updated gameState to the client. This function also closes the connection with the client once the client disconnects from the server.

	Parameters
		----------
		client_socket  : class<obj>, socket
            Socket which stores the connection of the client with the server. Used to send/retrieve game state to/from the server.

		played_id : integer
			The ID assigned to the connected client.

		client_address : tupple(string,int)
			Tupple containing the IP Address and the port of the connected client.

		gameState : Dictionary
            This dictionary contains complete information about all the clients connected to the server.

	Return
		----------
		None
	"""
	player_name = client_socket.recv(20).decode()
	print(f"Client {player_name} with IP: {client_address[0]} has joined the game!")
	pos = starting_pos(gameState)
	client_socket.send(str(player_id).encode())

	new_player_attributes = {
		"color" : random.choice(COLOR),
		"x" : pos[0],
		"y" : pos[1],
		"name" : player_name,
		"radius" : 0
	}
	gameState.connectedClients[int(player_id)] = new_player_attributes
	
	while True:
		if gameState.running:
			gameState.cur_time = round(time.time()-gameState.start_time)

			if gameState.cur_time >= gameState.total_time:
				gameState.running = False
			else:
				if gameState.cur_time // gameState.radius_loss_time == gameState.next_loss:
					reduce_radius(gameState)
					gameState.next_loss+=1  
		try:
			client_data = client_socket.recv(40)

			client_data = client_data.decode()
			splitted_data = client_data.split(" ")

			if splitted_data[0] == "gameState":
				send_data = json.dumps(gameState.__dict__)

			else:
				player_x = int(splitted_data[1])
				player_y = int(splitted_data[2])
				#Update the game state
				gameState.connectedClients[player_id]["x"] = player_x
				gameState.connectedClients[player_id]["y"] = player_y

				if gameState.running:
					player_eats_food(gameState)
					player_eats_player(gameState)

				if len(gameState.food_info) < 150:
					generate_food(random.randrange(100,150),gameState)

				send_data = json.dumps(gameState.__dict__)

			# send data back to clients
			encoded_data = send_data.encode()
			client_socket.send(encoded_data)

		except Exception as e:
			print(e)
			break

		time.sleep(0.005) 

	print(f"Client {player_name} with IP: {client_address[0]} has DISCONNECTED!!")
	del gameState.connectedClients[player_id]
	if(len(gameState.connectedClients)==0):
		gameState.running = False
	client_socket.close() 

def run_server(server_ip = ""):
	"""
	This function is responsible for starting the server on the provided IP address.

	Parameters
		----------
		server_ip : string
			The IP address of the host machine on which to start the server.

	Return
		----------
		None
	"""

	if(server_ip==""):
		print("Server IP not provided, couldn't start the server! ")
		sys.exit()  

	server = ServerConnection(server_ip)
	try:
		server.create_connection()
	except:
		print("Invalid IP entered for running the server! Please check your private IP and enter the correct one!")
		sys.exit()

	server_socket = server.server_socket
	gameState = GameState()

	generate_food(random.randrange(200,250),gameState)

	print("Waiting for players to connect!")

	while True:
		client_socket,client_address = server_socket.accept()

		if gameState.running == False:
			resetGameState(gameState)
		
		gameState.client_id+=1
		start_new_thread(connect_client,(client_socket,gameState.client_id,client_address,gameState))


class Button():
	def __init__(self, image, pos, text_input, font, base_color, hovering_color):
		self.image = image
		self.x_pos = pos[0]
		self.y_pos = pos[1]
		self.font = font
		self.base_color, self.hovering_color = base_color, hovering_color
		self.text_input = text_input
		self.text = self.font.render(self.text_input, True, self.base_color)
		if self.image is None:
			self.image = self.text
		self.rect = self.image.get_rect(center=(self.x_pos, self.y_pos))
		self.text_rect = self.text.get_rect(center=(self.x_pos, self.y_pos))

	def update(self, screen):
		if self.image is not None:
			screen.blit(self.image, self.rect)
		screen.blit(self.text, self.text_rect)

	def checkForInput(self, position):
		if position[0] in range(self.rect.left, self.rect.right) and position[1] in range(self.rect.top, self.rect.bottom):
			return True
		return False

	def changeColor(self, position):
		if position[0] in range(self.rect.left, self.rect.right) and position[1] in range(self.rect.top, self.rect.bottom):
			self.text = self.font.render(self.text_input, True, self.hovering_color)
		else:
			self.text = self.font.render(self.text_input, True, self.base_color)


import random

def populate_colors(n):
    colors = []
    for i in range(n):
        colors.append([random.randrange(0,255),random.randrange(0,255),random.randrange(0,255)])
    return colors

FPS = 30
SERVER_IP = "192.168.1.2"
SERVER_PORT = 12345
WINDOW_WIDTH = 1500
WINDOW_HEIGHT = 800
COLOR = populate_colors(10)








import argparse
from src import client
from src import server


def runServer():
    parser = argparse.ArgumentParser(
        description="Start the server on a private LAN on port 12345"
    )
    parser.add_argument(
       '-IP', '--IP', type=str,help="IP address to start the server on"
    )
    
    args = parser.parse_args()
    
    server.run_server(args.IP)

def runClient():
    parser = argparse.ArgumentParser(
        description="Connect your game to an existing server on the LAN! If the server is not started, start the server first!"
    )
    parser.add_argument(
       '-n', '--name', type=str,help="Client Name"
    )
    parser.add_argument(
       '-IP', '--IP', type=str,help="IP address of the server"
    )
    
    args = parser.parse_args()
    
    client.run_client(args.name,args.IP)





class GameState():
    """
	This class maintains the state of the game. 

     Parameters
        ----------
        connectedClients : Dictionary
            This stores all the connected clients' information as <client_id, information>
        client_id : integer
            Starting counter of the ID to be assigned to the clients connecting to the server.
        food_info : list[list[int,int,tupple(int,int,int)]]
            Contains a list of the circles/food generated on the grid.
        running : bool
            Maintains the current status of game lobby.
        cur_time : integer
            Counter to track the time elapsed since the game lobby started.
        total_time : integer
            Total time the game lobby will last for.
        start_time : integer 
            Time when the first client connected to the lobby
        food_radius : integer
            Radius of the circles rendered on the grid as food.
        player_radius : integer
            Initial radius of the player
        radius_loss_time : integer
            Decrease mass once every radius_loss_time seconds
        player_speed : integer
            Initial player speed

        


	"""
    def __init__(self):
        self.connectedClients={}
        self.client_id = 0
        self.food_info=[]
        self.running = False
        self.start_time = 0
        self.total_time = 120
        self.cur_time = 0
        self.food_radius = 5
        self.player_radius = 10
        self.radius_loss_time = 6
        self.next_loss = 1
        self.player_speed = 8
        


import setuptools

# with open("README.md", "r", encoding="utf-8") as fhand:
#     long_description = fhand.read()

setuptools.setup(
    name="eatemall",
    version="0.0.15",
    author="Jaskaran_Singh/Harsh_Singh",
    author_email="jaskaran_malhotra@yahoo.in",
    description="(CLI package to run the server and connect a client to play a multiplayer game!)",
    url="https://github.com/Jaskaran96/MultiplayerGame",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    install_requires=["pygame"],
    packages=setuptools.find_packages(),
    python_requires=">=3.6",
    include_package_data=True,
    package_data={'': ['./src/assets/*.png','./src/assets/*.ttf']},
    entry_points={
        "console_scripts": [
            "eatemall-server = src.cli:runServer",
            "eatemall-client = src.cli:runClient",
        ]
    }
)


import pygame
from src.client_connection import Connection
from src.button import Button
from src.constants import *
import sys
import os
import pathlib


def process_time(time):
    """
    This function will convert the time in integer to it's formatted string

    Parameters
        ----------
        time : int
            The current round time counter - Maintained by the server and passed through Game State
    Return
        ----------
        convertedTime : string
            returns the converted time of the current round time counter(Maintained by the server and passed through Game State) into formatted string that is to be displayed on the UI
    """
    minutes = str(time//60)
    seconds = str(time%60)
    if(int(seconds)<10):
        seconds = "0"+seconds
    convertedTime = minutes+":"+seconds
    return convertedTime

def HUD(gameState,player_id,display_window):
    """
    This function will render the information (which is extracted from the GameState) about a client on the pygame display_window

    Parameters
        ----------
        gameState : Dictionary
            This dictionary is received from the server and contains complete information about all the clients connected to the server.
    
        player_id : integer
            The ID of the client whose information is to be fetched from the gameState dictionary. 

        display_window : class<obj>, pygame
            The window that will render all the objects on the screen

    Return
        ----------
        None
    """
    LEADERBOARD_FONT = pygame.font.Font(pathlib.Path.joinpath(pathlib.Path(__file__).parent.resolve(),"assets\Facon.ttf"),20)
    ROUND_TIME_FONT = pygame.font.Font(pathlib.Path.joinpath(pathlib.Path(__file__).parent.resolve(),"assets\Facon.ttf"),30)
    SCORE_FONT = pygame.font.Font(pathlib.Path.joinpath(pathlib.Path(__file__).parent.resolve(),"assets\Facon.ttf"),26)
    
    sorted_by_radius = sorted(gameState["connectedClients"], key=lambda x: gameState["connectedClients"][x]["radius"])
    rev_sorted_players = list(reversed(sorted_by_radius))

    display_window.fill((255,255,255)) 
    
    for food_item in gameState["food_info"]:
        pygame.draw.circle(display_window, food_item[2], (food_item[0], food_item[1]), gameState["food_radius"])
    
    text = ROUND_TIME_FONT.render("Round Time: " + process_time(gameState["cur_time"]), 1, (0,0,0))
    display_window.blit(text,(10,10))

    # draw score
    text = ROUND_TIME_FONT.render("Score: " + str(round(gameState["connectedClients"][player_id]["radius"])),1,(0,0,0))
    display_window.blit(text,(10,15 + text.get_height()))

    title_text = ROUND_TIME_FONT.render("Leaderboard", True, (0,0,0))
    start_y = 50
    x = WINDOW_WIDTH - title_text.get_width() - 10
    display_window.blit(title_text, (x, 5))
    
    for player_id in sorted_by_radius:
        value = gameState["connectedClients"][player_id]
        pygame.draw.circle(display_window, value["color"], (value["x"], value["y"]), gameState["player_radius"] + round(value["radius"]))
        render_text = LEADERBOARD_FONT.render(value["name"], True, (0,0,0))
        display_window.blit(render_text , (value["x"] - text.get_width()/2 + 15, value["y"] - render_text.get_height()/2))

    run = min(len(gameState["connectedClients"]), 3)
    for count, i in enumerate(rev_sorted_players[:run]):
        text = SCORE_FONT.render(str(count+1) + ". " + str(gameState["connectedClients"][i]["name"]) + "  " + str(round(gameState["connectedClients"][i]["radius"])), True, (0,0,0))
        display_window.blit(text, (x, start_y + count * 40))


def quitGame(gameState,display_window):
    """
    This function will select the client/player with the highest score among all the clients. This will run once the round is over and render the winner on all the client's screen.

    Parameters
        ----------
        gameState : Dictionary
            This dictionary is received from the server and contains complete information about all the clients connected to the server.

        display_window : class<obj>, pygame
            The window that will render all the objects on the screen

    Return
        ----------
        None
    """
    sorted_by_radius = sorted(gameState["connectedClients"], key=lambda x: gameState["connectedClients"][x]["radius"])

    winner = sorted_by_radius[len(sorted_by_radius) - 1]
    winnerName = gameState["connectedClients"][winner]["name"]

    TIME_FONT = pygame.font.Font(pathlib.Path.joinpath(pathlib.Path(__file__).parent.resolve(),"assets\Facon.ttf"),60)
    
    text = TIME_FONT.render(f"Player {winnerName} has won the game!", 1, (0,0,0))
    display_window.blit(text,(100,WINDOW_HEIGHT//2 - 200))
    
    QUIT_BUTTON = Button(image=pygame.image.load(pygame.font.Font(pathlib.Path.joinpath(pathlib.Path(__file__).parent.resolve(),"quit.png"))), pos=(750, 450), 
                            text_input="QUIT", font=get_font(75), base_color="#d7fcd4", hovering_color="White")
    MOUSE_POS = pygame.mouse.get_pos()
    QUIT_BUTTON.changeColor(MOUSE_POS)
    QUIT_BUTTON.update(display_window)
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            pygame.quit()
            sys.exit()
        if event.type == pygame.MOUSEBUTTONDOWN:
            if QUIT_BUTTON.checkForInput(MOUSE_POS):
                pygame.quit()
                sys.exit()
    
def update_pos(gameState,player_id):
    """
    This function will capture the key presses made by a client to change it's position and update the gameState to incorporate these changes for a client.

    Parameters
        ----------
        gameState : Dictionary
            This dictionary is received from the server and contains complete information about all the clients connected to the server.
    
        player_id : integer
            The ID of the client whose information is to be fetched from the gameState dictionary. 

    Return
        ----------
        data : string
            This string is sent to the server along with the updated positions of the client.
    """
    data = ""
    keys = pygame.key.get_pressed()
    speed = gameState["player_speed"] - round(gameState["connectedClients"][player_id]["radius"]/14)
    speed = max(1,speed)

    if keys[pygame.K_UP] or keys[pygame.K_w]:
        if gameState["connectedClients"][player_id]["y"] - speed - gameState["player_radius"] - gameState["connectedClients"][player_id]["radius"] >= 0:
            gameState["connectedClients"][player_id]["y"] = gameState["connectedClients"][player_id]["y"] - speed

    if keys[pygame.K_DOWN] or keys[pygame.K_s]:
        if gameState["connectedClients"][player_id]["y"] + speed + gameState["player_radius"] + gameState["connectedClients"][player_id]["radius"] <= WINDOW_HEIGHT:
            gameState["connectedClients"][player_id]["y"] = gameState["connectedClients"][player_id]["y"] + speed

    if keys[pygame.K_LEFT] or keys[pygame.K_a]:
        if gameState["connectedClients"][player_id]["x"] - speed - gameState["player_radius"] - gameState["connectedClients"][player_id]["radius"] >= 0:
            gameState["connectedClients"][player_id]["x"] = gameState["connectedClients"][player_id]["x"] - speed

    if keys[pygame.K_RIGHT] or keys[pygame.K_d]:
        if gameState["connectedClients"][player_id]["x"] + speed + gameState["player_radius"] + gameState["connectedClients"][player_id]["radius"] <= WINDOW_WIDTH:
            gameState["connectedClients"][player_id]["x"] = gameState["connectedClients"][player_id]["x"] + speed


    data = "changepos " + str(gameState["connectedClients"][player_id]["x"]) + " " + str(gameState["connectedClients"][player_id]["y"])

    return data

def game(client_socket,display_window):
    
    """
    This function runs the main game loop, sending client updates to the server and fetching the updated state from the server. It calls the HUD to draw the updates on the screen. It also calls quitGame once the round is over.

    Parameters
        ----------
        client_socket  : class<obj>, socket
            Socket which stores the connection with the server. Used to send/retrieve game state to/from the server.

        display_window : class<obj>, pygame
            The window that will render all the objects on the screen

    Return
        ----------
        None
    """
    player_id = client_socket.player_id
    gameState = client_socket.send_normal("gameState")

    clock = pygame.time.Clock()

    running = True

    while running:

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False

        clock.tick(FPS)

        data = update_pos(gameState,player_id)
        gameState = client_socket.send_normal(data)
        
        HUD(gameState,player_id,display_window)
        
        if gameState["total_time"] == gameState["cur_time"]:
            quitGame(gameState,display_window)

        pygame.display.update()

    client_socket.client_socket.close()
    pygame.quit()
    quit()

def get_font(size): 
    pygame.font.Font(pathlib.Path.joinpath(pathlib.Path(__file__).parent.resolve(),"quit.png"))
    return pygame.font.Font(pathlib.Path.joinpath(pathlib.Path(__file__).parent.resolve(),"assets\menu.ttf"),60)

def main_menu(player_name,display_window,IP_addr):
    """
    This function renders the landing screen for the game. It initiates the connection with the server. The client/player can chose to start/quit the game from this landing screen.

    Parameters
        ----------
        player_name : String
            The name of the client that will be displayed on the screen.
    
        display_window : class<obj>, pygame
            The window that will render all the objects on the screen 

        IP_addr : String
            The IP address of the server to which the client connects.

    Return
        ----------
        NONE
    """
   
    try:
        client_socket = Connection(player_name,IP_addr)
    except:
        print("QUTTING!!! Invalid IP ADDRESS entered or the server is down!")
        sys.exit()

    BG = pygame.image.load(pathlib.Path.joinpath(pathlib.Path(__file__).parent.resolve(),"assets\Background.png"))
    while True:
        display_window.blit(BG, (0, 0))

        MENU_MOUSE_POS = pygame.mouse.get_pos()

        MENU_TEXT = get_font(100).render("MAIN MENU", True, "#b68f40")
        MENU_RECT = MENU_TEXT.get_rect(center=(750, 100))
        WELCOME_FONT = pygame.font.Font(pathlib.Path.joinpath(pathlib.Path(__file__).parent.resolve(),"assets\Facon.ttf"),60)
    
        text = WELCOME_FONT.render(f"Welcome {player_name}!", 1, (255,255,255))
        display_window.blit(text,(400,250))
        PLAY_BUTTON = Button(image=pygame.image.load(pathlib.Path.joinpath(pathlib.Path(__file__).parent.resolve(),"assets\play.png")), pos=(750, 400), 
                            text_input="PLAY", font=get_font(75), base_color="#d7fcd4", hovering_color="White")
        QUIT_BUTTON = Button(image=pygame.image.load(pathlib.Path.joinpath(pathlib.Path(__file__).parent.resolve(),"assets\quit.png")), pos=(750, 550), 
                            text_input="QUIT", font=get_font(75), base_color="#d7fcd4", hovering_color="White")
        
        display_window.blit(MENU_TEXT, MENU_RECT)
        
        buttons = [PLAY_BUTTON, QUIT_BUTTON]

        for button in buttons:
            button.changeColor(MENU_MOUSE_POS)
            button.update(display_window)
        
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()
            if event.type == pygame.MOUSEBUTTONDOWN:
                if PLAY_BUTTON.checkForInput(MENU_MOUSE_POS):
                    game(client_socket,display_window)
                if QUIT_BUTTON.checkForInput(MENU_MOUSE_POS):
                    pygame.quit()
                    sys.exit()

        pygame.display.update()

def run_client(player_name="",IP_addr=""):
    """
    This function is responsible for starting the client. The name of the client and the server IP address to which the client wants to connect is provided.

    Parameters
        ----------
        player_name : String
            The name of the client that will be displayed on the screen.
    
        IP_addr : String
            The IP address of the server to which the client connects.

    Return
        ----------
        NONE
    """
    pygame.font.init()

    display_window = pygame.display.set_mode((WINDOW_WIDTH,WINDOW_HEIGHT))
    pygame.display.set_caption("Eat em ALL!")
    
    if(player_name == ""):
        print("Client name not provided, quitting the client!")
        pygame.quit()
        sys.exit()

    if(IP_addr == ""):
        print("IP Address not provided, quitting the client!")
        pygame.quit()
        sys.exit()

    main_menu(player_name,display_window,IP_addr)


import json
import socket
from src.constants import SERVER_IP,SERVER_PORT

class Connection():

    def __init__(self,playerName,IP=""):
        self.playerName = playerName

        if(IP != ""):
            self.ip_addr = IP
        else:
            self.ip_addr = SERVER_IP
            
        self.destination_port = SERVER_PORT
        self.socket_address = (self.ip_addr,self.destination_port)
        self.client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        #Connect to the client
        self.client_socket.connect(self.socket_address)
        self.client_socket.send(self.playerName.encode())
        self.player_id = self.client_socket.recv(8).decode()

    def send_normal(self,message):

        self.client_socket.send(message.encode())
        data = self.client_socket.recv(32768)
        print(type(data))
        return json.loads(data)

    def send_json(self,message):

        self.client_socket.send(json.dumps(message).encode())
        data = self.client_socket.recv(32768)
        return json.loads(data)

